﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace FATX_Browser.FATX
{
    public class FATXDrive
    {
        public string DumpPath = "";
        FATX.Misc r = new FATX.Misc();

        private int deviceID;
        private long dSize;
        private string sizeConverted;
        public FATX.Info.DriveType DriveType;
        public Folder[] Partitions;
        private IO.HDDFATX io;
        public System.IO.BinaryReader br;
        public System.IO.BinaryWriter bw;

        private bool dump;
        /// <summary>
        /// Provides information about a specific Xbox 360 Drive
        /// </summary>
        public FATXDrive(int ID, Info.DriveType dtype)
        {
            deviceID = ID;
            DriveType = dtype;
            ReadData();
        }

        public FATXDrive(bool Dump, string dumpPath, Info.DriveType dtype)
        {
            DriveType = dtype;
            IsDump = Dump;
            DumpPath = dumpPath;
            ReadData();
        }

        internal System.IO.BinaryReader GetIO()
        {
            if (DriveType == Info.DriveType.HDD)
            {
                try
                {
                    io.Close();
                    br.Close();
                }
                catch { }
                io = new IO.HDDFATX(false, null);
                br = io.br_diskReader(DeviceID);
            }
            else if (DriveType == Info.DriveType.Backup)
            {
                try
                {
                    io.Close();
                    br.Close();
                }
                catch { }
                io = new IO.HDDFATX(true, DumpPath);
                br = io.br_dumpReader();
            }
            return br;
        }

        internal System.IO.BinaryWriter GetWriterIO()
        {
            if (DriveType == Info.DriveType.HDD)
            {
                try
                {
                    io.Close();
                    br.Close();
                    bw.Close();
                }
                catch { }
                io = new IO.HDDFATX(false, null);
                bw = io.bw_diskWriter(DeviceID);
            }
            else if (DriveType == Info.DriveType.Backup)
            {
                try
                {
                    io.Close();
                    br.Close();
                    bw.Close();
                }
                catch { }
                io = new IO.HDDFATX(true, DumpPath);
                bw = io.bw_dumpWriter();
            }
            return bw;
        }

        public bool IsDump
        {
            get { return dump; }
            set { dump = value; }
        }

        private void ReadData()
        {
            GetIO();
            if (IsDump)
            {
                dSize = io.GetDriveSize();
            }
            else
            {
                dSize = io.GetDriveSize(DeviceID);
            }
            sizeConverted = r.ByteConversion(dSize);
            Partitions = new FATX.Partitions().Get(this);
        }

        /// <summary>
        /// ID for the device (used for the Win32 API)
        /// </summary>
        public int DeviceID
        {
            get { return deviceID; }
            set { deviceID = value; }
        }

        /// <summary>
        /// Drive size in bytes
        /// </summary>
        public long DriveSize
        {
            get { return dSize; }
        }

        /// <summary>
        /// Drive size converted to KB/MB/GB
        /// </summary>
        public string DriveSizeConverted
        {
            get { return sizeConverted; }
        }

        public bool Close()
        {
            try
            {
                io.Close();
                bw.Close();
                br.Close();
                return true;
            }
            catch { return true; }
        }
    }
}
