﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using FATX_Browser.Win32;
using Microsoft.Win32.SafeHandles;
using System.IO;

namespace FATX_Browser.FATX
{
    class IO
    {

        public class HDDFATX
        {
            protected internal System.IO.Stream fs;
            protected internal System.IO.BinaryReader br;
            protected internal System.IO.BinaryWriter bw;
            SafeFileHandle diskDrive;
            private bool Dump;
            private string DumpPath;

            public HDDFATX(bool dump, string dumpPath)
            {
                if (dump)
                {
                    Dump = dump;
                    DumpPath = dumpPath;
                }
            }

            #region Get FATX Drives

            /// <summary>
            /// Returns an array of valid FATXDrives
            /// </summary>
            public FATX.FATXDrive[] GetFATXDrives(int range)
            {
                List<FATX.FATXDrive> driveList = new List<FATX.FATXDrive>();
                for (int i = 0; i < range; i++)
                {
                    try
                    {
                        //Start reading the physical drive
                        fs = fs_diskStream(i);
                        br = br_diskReader(i);
                        //Seek to the FATX partition
                        //try
                        //{
                            br.BaseStream.Position = (long)FATX.Info.HDDFATX.Partitions.Data;
                            //Read the header
                            byte[] header = br.ReadBytes(0x4);
                            //Convert the header to a string, check if it's fatx
                            if (Encoding.ASCII.GetString(header) == "XTAF")
                            {
                                FATX.FATXDrive drive = new FATXDrive(i, FATX.Info.DriveType.HDD);
                                driveList.Add(drive);
                            }
                        //}
                        //catch{ }
                        br.Close();
                        fs.Close();
                        diskDrive.Close();
                        bool closed = diskDrive.IsClosed;
                    }
                    catch { fs.Close(); br.Close(); diskDrive.Close(); }
                }
                return driveList.ToArray();
            }

            #endregion

            #region Get Drive Size

            /// <summary>
            /// Returns the size of a HDD
            /// </summary>
            public long GetDriveSize(int deviceNumber)
            {
                API api = new API();
                var diskGeo = new API.DISK_GEOMETRY();
                bool result = api.GetDriveGeometry(ref diskGeo, deviceNumber, CreateHandle(deviceNumber));
                return diskGeo.DiskSize;
            }

            public long GetDriveSize()
            {
                return fs_dumpStream().Length;
            }

            #endregion

            #region File Stream/Binary Reader
            /// <summary>
            /// Stream for the disk
            /// </summary>
            protected internal System.IO.Stream fs_diskStream(int deviceNumber)
            {
                CreateHandle(deviceNumber);
                fs = new System.IO.FileStream(diskDrive, System.IO.FileAccess.ReadWrite);
                return fs;
            }

            protected internal System.IO.Stream fs_dumpStream()
            {
                try { fs.Close(); }
                catch { }
                fs = new System.IO.FileStream(DumpPath, FileMode.Open);
                return fs;
            }

            /// <summary>
            /// Returns a binary reader to read the disk
            /// </summary>
            protected internal System.IO.BinaryReader br_diskReader(int deviceNumber)
            {
                br = new System.IO.BinaryReader(fs_diskStream(deviceNumber));
                return br;
            }

            protected internal System.IO.BinaryReader br_dumpReader()
            {
                try { br.Close(); }
                catch { }
                br = new BinaryReader(fs_dumpStream());
                return br;
            }

            /// <summary>
            /// Returns a binary writer to write to the disk
            /// </summary>
            protected internal System.IO.BinaryWriter bw_diskWriter(int deviceNumber)
            {
                bw = new BinaryWriter(fs_diskStream(deviceNumber));
                return bw;
            }

            protected internal System.IO.BinaryWriter bw_dumpWriter()
            {
                bw = new BinaryWriter(fs_dumpStream());
                return bw;
            }

            /// <summary>
            /// Creates and returns a new handle for reading a drive
            /// </summary>
            public SafeFileHandle CreateHandle(int deviceNumber)
            {
                try { diskDrive.Close(); }
                catch{};
                diskDrive = API.CreateFile(@"\\.\PhysicalDrive" + deviceNumber.ToString(), System.IO.FileAccess.ReadWrite, System.IO.FileShare.ReadWrite, IntPtr.Zero, System.IO.FileMode.Open, API.FlagsAndAttributes.Device | API.FlagsAndAttributes.NoBuffering | API.FlagsAndAttributes.Write_Through, IntPtr.Zero);
                return diskDrive;
            }

            #endregion

            /// <summary>
            /// Closes the readers/handles
            /// </summary>
            public bool Close()
            {
                try
                {
                    fs.Close();
                    br.Close();
                    bw.Close();
                    diskDrive.Close();
                    return true;
                }
                catch { return true; }
            }
        }
    }

    #region CLKHDDIO (Not Used)
    class CLKHDDIO
    {
        private Stream str;
        private bool closed = false;
        public CLKHDDIO(Stream stream)
        {
            str = stream;
        }

        private bool CheckStream()
        {
            try { byte[] n = new byte[1]; IOStream.Read(n, 0x0, 1); }
            catch { closed = true; return false; }
            return true;
        }

        public Stream IOStream
        {
            get { return str; }
        }

        public byte[] ReadBytes(int Size)
        {
            //The stream is closed...
            if (!IsClosed)
            {
                throw new Exception("Stream is closed");
            }
            byte[] rVal = new byte[Size];
            IOStream.Read(rVal, 0, Size);
            return rVal;
        }

        public virtual long Position
        {
            get { return str.Position; }
            set { str.Seek(value, SeekOrigin.Begin); }
        }

        #region Reading

        public byte ReadByte()
        {
            byte[] rVal = ReadBytes(1);
            return rVal[0];
        }

        #region Signed Values
        public long ReadInt64()
        {
            byte[] bytes = ReadBytes(8);
            Array.Reverse(bytes);
            return BitConverter.ToInt64(bytes, 0x0);
        }

        public long ReadInt64(bool BigEndian)
        {
            if (BigEndian)
            {
                byte[] bytes = ReadBytes(8);
                return BitConverter.ToInt64(bytes, 0x0);
            }
            return ReadInt64();
        }

        public int ReadInt32()
        {
            byte[] bytes = ReadBytes(4);
            return BitConverter.ToInt32(bytes, 0);
        }

        public int ReadInt32(bool BigEndian)
        {
            if (BigEndian)
            {
                return BitConverter.ToInt32(ReadBytes(4), 0);
            }
            return ReadInt32();
        }

        public int ReadInt16()
        {
            byte[] bytes = ReadBytes(2);
            Array.Reverse(bytes);
            return BitConverter.ToInt16(bytes, 0);
        }

        public int ReadInt16(bool BigEndian)
        {
            if (BigEndian)
            {
                return BitConverter.ToInt16(ReadBytes(2), 0);
            }
            return ReadInt16();
        }
        #endregion

        #region Unsigned Values

        public ulong ReadUInt64()
        {
            byte[] bytes = ReadBytes(8);
            Array.Reverse(bytes);
            return BitConverter.ToUInt64(bytes, 0x0);
        }

        public ulong ReadUInt64(bool BigEndian)
        {
            if (BigEndian)
            {
                byte[] bytes = ReadBytes(8);
                return BitConverter.ToUInt64(bytes, 0x0);
            }
            return ReadUInt64();
        }

        public uint ReadUInt32()
        {
            byte[] bytes = ReadBytes(4);
            return BitConverter.ToUInt32(bytes, 0);
        }

        public uint ReadUInt32(bool BigEndian)
        {
            if (BigEndian)
            {
                return BitConverter.ToUInt32(ReadBytes(4), 0);
            }
            return ReadUInt32();
        }

        public uint ReadUInt16()
        {
            byte[] bytes = ReadBytes(2);
            Array.Reverse(bytes);
            return BitConverter.ToUInt16(bytes, 0);
        }

        public uint ReadUInt16(bool BigEndian)
        {
            if (BigEndian)
            {
                return BitConverter.ToUInt16(ReadBytes(2), 0);
            }
            return ReadUInt16();
        }

        #endregion

        #endregion

        #region Writing

        public bool Write(string value)
        {
            byte[] v = Encoding.ASCII.GetBytes(value);
            try { IOStream.Write(v, 0, 0); }
            catch { return false; }
            return true;
        }

        #endregion

        public bool Close()
        {
            try { IOStream.Close(); }
            catch { return false; }
            return true;
        }

        public bool IsClosed
        {
            get { return closed; }
        }
    }
    #endregion
}
