﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using FATX_Browser.FATX;

namespace FATX_Browser
{
    public partial class Main : Form
    {
        internal FATXDrive drive;
        TreeNode lastClickedNode;
        bool Loaded = false;
        Misc misc = new Misc();
        int clickedColumn = int.MaxValue - 0x1;
        private List<string> visitedFolders = new List<string>();
        public Main()
        {
            #region Startup Stuff
            InitializeComponent();
            //Get our misc class so that we can get our imagelist
            Misc m = new Misc();
            //Create our imagelist
            ImageList il = m.ListForFATX;

            //Assign the imagelist to both the listview & treeview
            treeView1.ImageList = il;
            listView1.SmallImageList = il;

            //Assign the listview a contextmenu
            listView1.ContextMenu = listview_ContextMenu;

            listView1.ColumnClick += new ColumnClickEventHandler(listView1_ColumnClick);
            treeView1.ContextMenu = treeview_ContextMenu;
            foreach (MenuItem mi in listview_ContextMenu.MenuItems)
            {
                mi.Enabled = false;
            }
            foreach (MenuItem mi in treeview_ContextMenu.MenuItems)
            {
                mi.Enabled = false;
            }
            foreach (MenuItem mi in menuItem9.MenuItems)
            {
                mi.Enabled = false;
            }

            treeView1.AfterLabelEdit += new NodeLabelEditEventHandler(treeView1_AfterLabelEdit);
            listView1.DoubleClick += new EventHandler(listView1_DoubleClick);
            #endregion
        }

        void listView1_DoubleClick(object sender, EventArgs e)
        {
            Entry E = (Entry)listView1.FocusedItem.Tag;
            foreach (TreeNode n in treeView1.SelectedNode.Nodes)
            {
                if (n.Text == E.Name)
                {
                    treeView1.SelectedNode = treeView1.SelectedNode.Nodes[n.Index];
                    ReadFolderData();
                }
            }
        }

        #region Treeview Editing
        /// <summary>
        /// Renames the folder on the drive - Checks if the node is a root/partition
        /// </summary>
        void treeView1_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            if (treeView1.SelectedNode != treeView1.Nodes[0])
            {
                bool canEdit = true;
                for (int i = 0; i < treeView1.Nodes[0].Nodes.Count - 1; i++)
                {
                    if (treeView1.SelectedNode == treeView1.Nodes[0].Nodes[i])
                    {
                        canEdit = false;
                        e.CancelEdit = true;
                        return;
                    }
                }
                if (canEdit)
                {
                    Folder f = (Folder)treeView1.SelectedNode.Tag;
                    Write w = new Write(xDrive);
                    bool result = w.Rename(f, e.Label);
                    if (result)
                    {
                        treeView1.SelectedNode.Tag = f;
                    }
                    else
                    {
                        e.CancelEdit = true;
                    }
                }
            }
            else
            {
                e.CancelEdit = true;
            }
        }
        #endregion

        #region Column Sorting for Listview
        /// <summary>
        /// Column Sorter
        /// </summary>
        void listView1_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            //Check if the last column clicked was this column
            if (clickedColumn == e.Column)
            {
                //Check the current sorting
                if (listView1.Sorting == SortOrder.Ascending)
                {
                    listView1.Sorting = SortOrder.Descending;
                }
                else
                {
                    listView1.Sorting = SortOrder.Ascending;
                }
            }
            else
            {
                clickedColumn = e.Column;
                listView1.Sorting = SortOrder.Ascending;
            }
        }
        #endregion

        internal FATXDrive xDrive
        {
            get { return drive; }
            set { drive = value; }
        }

        #region Show Forms
        /// <summary>
        /// Show about form
        /// </summary>
        private void ShowAbout_menuItem7_Click(object sender, EventArgs e)
        {
            About abt = new About();
            abt.ShowDialog();
        }

        private void ShowDeviceSelector_menuItem2_Click(object sender, EventArgs e)
        {
            DeviceSelector ds = new DeviceSelector(this);
            if (ds.ShowDialog() == DialogResult.OK)
            {
                Loaded = true;
                ReadDrive();
            }
        }
        #endregion

        /// <summary>
        /// Clears and resets the form
        /// </summary>
        private void Clear()
        {
            foreach (MenuItem mi in listview_ContextMenu.MenuItems)
            {
                mi.Enabled = false;
            }
            foreach (MenuItem mi in treeview_ContextMenu.MenuItems)
            {
                mi.Enabled = false;
            }
            foreach (MenuItem mi in menuItem9.MenuItems)
            {
                mi.Enabled = false;
            }
            listView1.Items.Clear();
            treeView1.Nodes[0].Nodes.Clear();
            treeView1.Nodes[0].Text = "Drive";
            Loaded = false;
            if (xDrive != null)
            {
                xDrive.Close();
                xDrive = null;
            }
        }

        public void ReadDrive()
        {
            foreach (MenuItem mi in listview_ContextMenu.MenuItems)
            {
                mi.Enabled = true;
            }
            foreach (MenuItem mi in treeview_ContextMenu.MenuItems)
            {
                mi.Enabled = true;
            }
            foreach (MenuItem mi in menuItem9.MenuItems)
            {
                mi.Enabled = true;
            }
            AddPartitionsToTree();
        }

        /// <summary>
        /// Adds Base Partitions to treeview
        /// </summary>
        private void AddPartitionsToTree()
        {
            //Create our folders (that will act as partitions) for each partition.
            Folder[] Partitions = new FATX.Partitions().Get(xDrive);
            //Set their EData
            for (int i = 0; i < Partitions.Length; i++)
            {
                Partitions[i].Name = Partitions[i].PartInfo.Name;
                Partitions[i].EData.StartingCluster = 1;
            }

            //Clear the subnodes
            treeView1.Nodes[0].Nodes.Clear();
            treeView1.Nodes[0].Text = "HDD " + xDrive.DriveSizeConverted;
            
            //Create our nodes
            List<TreeNode> TN = new List<TreeNode>();
            foreach (Folder f in Partitions)
            {
                TreeNode tn = new TreeNode(f.Name);
                tn.Tag = f;
                TN.Add(tn);
            }
            TreeNode[] nodeArray = TN.ToArray();
            treeView1.Nodes[0].Nodes.AddRange(nodeArray);
            treeView1.Nodes[0].Expand();
        }

        private void menuItem4_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// </summary>
        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            ReadFolderData();
        }

        private void ReadSubFolders(ref TreeNode node)
        {
            foreach (Folder f in ((Folder)node.Tag).SubFolders())
            {
                TreeNode subNode = new TreeNode(f.Name);
                subNode.Tag = f;
                node.Nodes.Add(subNode);
            }
        }

        private void ReadFolderData()
        {
            try
            {
                if (Loaded && treeView1.SelectedNode != treeView1.Nodes[0])
                {
                    //If they just loaded this node
                    if (lastClickedNode == treeView1.SelectedNode)
                    {
                        return;
                    }
                    //Set the last clicked nocde
                    lastClickedNode = treeView1.SelectedNode;
                    //Clear our listview
                    listView1.Items.Clear();
                    //Create a list of our nodes to add
                    List<TreeNode> treeList = new List<TreeNode>();
                    //Create our itemlist as well
                    List<ListViewItem> itemList = new List<ListViewItem>();
                    //Create our reader
                    Folder fffffff = (Folder)lastClickedNode.Tag;
                    FATX.Entries read = new FATX_Browser.FATX.Entries(fffffff);
                    //Loop for each folder
                    Folder[] folders = ((Folder)lastClickedNode.Tag).SubFolders();
                    foreach (FATX.Folder f in folders)
                    {
                        bool exists = false;
                        foreach (TreeNode TN in treeView1.SelectedNode.Nodes)
                        {
                            //If the node already exists...
                            if (TN.Text == f.Name)
                            {
                                //The node exists
                                exists = true;
                            }
                        }
                        if (!exists)
                        {
                            //Create a new node with the folder name
                            TreeNode node = new TreeNode(f.Name, 0, 0);
                            //Create our treenode subitems (subfolders) and add them to
                            //the node

                            foreach (Folder g in f.SubFolders())
                            {
                                TreeNode subNode = new TreeNode(g.Name);
                                subNode.Tag = g;
                                //ReadSubFolders(ref subNode);
                                node.Nodes.Add(subNode);
                            }

                            //Set the tag
                            node.Tag = f;
                            //Add to the list
                            treeList.Add(node);
                        }

                        //Create our listview item
                        ListViewItem item = new ListViewItem(f.Name, 0);
                        //Set our tag
                        item.Tag = f;
                        //Create our "file type" subitem
                        ListViewItem.ListViewSubItem type = new ListViewItem.ListViewSubItem(item, "File Folder");
                        item.SubItems.Add(type);
                        //Add our item to the list
                        itemList.Add(item);
                    }

                    //For each file
                    File[] files = ((Folder)lastClickedNode.Tag).Files();
                    foreach (File f in files)
                    {

                        //Create a new listview item with the file name
                        ListViewItem item = new ListViewItem(f.Name, 1);
                        //Set the tag
                        item.Tag = f;
                        //Create our "file type" subitem
                        ListViewItem.ListViewSubItem type = new ListViewItem.ListViewSubItem(item, "File");
                        //Create our "size" subitem
                        ListViewItem.ListViewSubItem size = new ListViewItem.ListViewSubItem(item, misc.ByteConversion(f.Size));
                        item.SubItems.Add(type);
                        item.SubItems.Add(size);
                        //Add to the list
                        itemList.Add(item);
                    }
                    //Add our list of folders to the treeview node
                    treeView1.SelectedNode.Nodes.AddRange(treeList.ToArray());
                    //Add our list of files and folders to the listview
                    listView1.Items.AddRange(itemList.ToArray());
                    //Extend the node
                    treeView1.SelectedNode.Expand();
                    statusBar1.Text = treeView1.SelectedNode.FullPath;
                }
            }
            catch { MessageBox.Show("Error reading from drive"); Clear(); }
        }

        /// <summary>
        /// Extracts an object from the listview
        /// </summary>
        private void ExtractListview_menuItem14_Click(object sender, EventArgs e)
        {
            SaveFileDialog ofd = new SaveFileDialog();
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                Extraction ex = new Extraction((File)listView1.FocusedItem.Tag, ofd.FileName);
                ex.ShowDialog();
            }
        }

        private void ExtractForm(object sender, EventArgs e)
        {
            TreeNode n = treeView1.SelectedNode;
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            if (fbd.ShowDialog() == DialogResult.OK)
            {
                Extraction Ex = new Extraction((Folder)n.Tag, fbd.SelectedPath);
                Ex.ShowDialog();
            }
        }

        /// <summary>
        /// Opens dialog to allow user to save backup of drive
        /// </summary>
        private void ShowBackup_menuItem10_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Filter = "Backup File(*.bin)|*.bin";
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                Backup b = new Backup(sfd.FileName, xDrive);
                b.ShowDialog();
            }
        }

        private void menuItem22_Click(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode != treeView1.Nodes[0])
            {
                bool canEdit = true;
                for (int i = 0; i < treeView1.Nodes[0].Nodes.Count; i++)
                {
                    if (treeView1.SelectedNode == treeView1.Nodes[0].Nodes[i])
                    {
                        canEdit = false;
                        return;
                    }
                }
                if (canEdit)
                {
                    treeView1.SelectedNode.BeginEdit();
                }
            }
        }

        private void menuItem21_Click(object sender, EventArgs e)
        {

        }

        /// <summary>
        /// Opens drive backup
        /// </summary>
        private void OpenBackup_menuItem16_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                xDrive = new FATXDrive(true, ofd.FileName, Info.DriveType.Backup);
                Loaded = true;
                ReadDrive();
            }
        }

        private void menuItem3_Click(object sender, EventArgs e)
        {
            Clear();
        }

        private void NodeFromPath()
        {

        }

        private void menuItem18_Click(object sender, EventArgs e)
        {
            Write w = new Write(xDrive);
            if (listView1.FocusedItem != null && ((Entry)listView1.FocusedItem.Tag).IsFolder)
            {
                Folder f = (Folder)listView1.FocusedItem.Tag;
                long throwaway = 0;
                long bleeh = 0;
                w.Delete(f, ref throwaway, ref bleeh);
                foreach (TreeNode n in treeView1.SelectedNode.Nodes)
                {
                    if (n.Text == f.Name)
                    {
                        treeView1.SelectedNode.Nodes.Remove(n);
                        break;
                    }
                }
            }
        }

        private void menuItem26_Click(object sender, EventArgs e)
        {
            Write w = new Write(xDrive);
            Folder f = (Folder)treeView1.SelectedNode.Tag;
            w.NewFolder("cock", ref f);
        }
    }
}
