﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace FATX_Browser
{
    public partial class DeviceSelector : Form
    {
        System.Threading.Thread t;
        Main m;
        public DeviceSelector(Main ma)
        {
            InitializeComponent();
            listView1.DoubleClick += new EventHandler(listView1_DoubleClick);
            t = new System.Threading.Thread(new System.Threading.ThreadStart(GetDrives));
            t.Start();
            m = ma;
            this.FormClosing += new FormClosingEventHandler(DeviceSelector_FormClosing);
        }

        void DeviceSelector_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                t.Abort();
            }
            catch { }
        }

        void listView1_DoubleClick(object sender, EventArgs e)
        {
            Ok.PerformClick();
        }

        private void GetDrives()
        {
            //The thread starts as soon as the device selector is created,
            //so just to make sure that we don't throw an error because the form
            //isn't there yet...
            if (!this.IsAccessible)
            {
                System.Threading.Thread.Sleep(500);
            }
            listView1.Invoke(new Bleeh(listView1.Items.Clear));
            //Update the label
            label1.Invoke(new UpdateText(updateLabel), new object[] { "Getting drives..." });
            //Create a new instance of the IO class
            FATX.IO.HDDFATX io = new FATX_Browser.FATX.IO.HDDFATX(false, null);
            //Get the drives
            FATX.FATXDrive[] physDrives = io.GetFATXDrives(10);
            //Add the drives to our listview if the array isn't null
            if (physDrives.Length != 0x0)
            {
                FATX.Misc r = new FATX_Browser.FATX.Misc();
                //For each index in our drives
                for (int i = 0; i < physDrives.Length; i++)
                {
                    //Get the physical drive length
                    long driveSize = io.GetDriveSize(2);
                    //Create a new listview item that will act as our "drive"
                    string Type = "Xbox 360 Hard Disk";
                    if (physDrives[i].DriveType == FATX.Info.DriveType.USB)
                    {
                        Type = "USB Memory Unit";
                    }
                    ListViewItem hdd = new ListViewItem(Type);
                    hdd.Tag = physDrives[i];
                    //Create our subitem
                    ListViewItem.ListViewSubItem id = null;
                    if (physDrives[i].DriveType == FATX.Info.DriveType.HDD)
                    {
                        id = new ListViewItem.ListViewSubItem(hdd, physDrives[i].DeviceID.ToString());
                    }
                    else
                    {
                        id = new ListViewItem.ListViewSubItem(hdd, physDrives[i].DumpPath.Remove(3));
                    }
                    //Create our size subitem
                    ListViewItem.ListViewSubItem size = new ListViewItem.ListViewSubItem(hdd, physDrives[i].DriveSizeConverted);
                    //Add the items to our listview
                    listView1.Invoke(new UpdateListView(AddToListview), new object[] { hdd, id, size });
                    //Update our label
                    label1.Invoke(new UpdateText(updateLabel), new object[] { "Done" });
                }
            }
                //The array IS null (no drives found)
            else
            {
                label1.Invoke(new UpdateText(updateLabel), new object[] { "No drives found" });
            }

            System.Threading.Thread.CurrentThread.Abort();
        }

        #region Other Functions
        private delegate void UpdateListView(ListViewItem item, ListViewItem.ListViewSubItem ID, ListViewItem.ListViewSubItem Size);

        private void AddToListview(ListViewItem item, ListViewItem.ListViewSubItem ID, ListViewItem.ListViewSubItem Size)
        {
            listView1.Items.Add(item);
            item.SubItems.Add(ID);
            item.SubItems.Add(Size);
        }

        private delegate void Bleeh();

        private delegate void UpdateText(string text);

        private void updateLabel(string text)
        {
            label1.Text = text;
        }
        #endregion

        private void button1_Click(object sender, EventArgs e)
        {
            if (listView1.FocusedItem == null)
            {
                //They didn't select a drive
                MessageBox.Show("Select a drive first!");
            }
                //They did select a drive
            else
            {
                //Set the device ID in the main form, load the drive, close this form
                m.xDrive = (FATX.FATXDrive)listView1.FocusedItem.Tag;
                this.Close();
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            try
            {
                t.Abort();
            }
            catch { }
            this.Close();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            t = new System.Threading.Thread(new System.Threading.ThreadStart(GetDrives));
            t.Start();
        }

        private void listView1_SelectedIndexChanged(object sender, EventArgs e)
        {
            
        }
    }
}
