﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using FATX_Browser.FATX;

namespace FATX_Browser
{
    public partial class Main : Form
    {
        bool Deletionmode = false;
        bool Beta = false;
        internal FATXDrive drive;
        TreeNode lastClickedNode;
        bool Loaded = false;
        Misc misc = new Misc();
        int clickedColumn = int.MaxValue - 0x1;
        private List<string> visitedFolders = new List<string>();
        public Main()
        {
            #region Startup Stuff
            InitializeComponent();
            this.FormClosing += new FormClosingEventHandler(Main_FormClosing);
            Update u = new FATX_Browser.Update();
            FATX_Browser.Update.UpdateInfo ui = new FATX_Browser.Update.UpdateInfo();
            ui = u.CheckForUpdates(new Uri("http://clkxu5.com/drivexplore/coolapplicationstuff.xml"));
            if (ui.Update)
            {
                UpdateForm uf = new UpdateForm(ui);
                if (uf.ShowDialog() == System.Windows.Forms.DialogResult.Yes)
                {
                    UpdateDownloadForm ud = new UpdateDownloadForm(ui);
                    ud.ShowDialog();
                }
            }

            //Get our misc class so that we can get our imagelist
            Misc m = new Misc();
            //Create our imagelist
            ImageList il = m.ListForFATX;

            //Assign the imagelist to both the listview & treeview
            treeView1.ImageList = il;
            listView1.SmallImageList = il;

            //Assign the listview a contextmenu
            listView1.ContextMenu = listview_ContextMenu;

            listView1.ColumnClick += new ColumnClickEventHandler(listView1_ColumnClick);
            treeView1.ContextMenu = treeview_ContextMenu;
            foreach (MenuItem mi in listview_ContextMenu.MenuItems)
            {
                mi.Enabled = false;
            }
            foreach (MenuItem mi in treeview_ContextMenu.MenuItems)
            {
                mi.Enabled = false;
            }
            foreach (MenuItem mi in menuItem9.MenuItems)
            {
                mi.Enabled = false;
            }

            treeView1.AfterLabelEdit += new NodeLabelEditEventHandler(treeView1_AfterLabelEdit);
            listView1.DoubleClick += new EventHandler(listView1_DoubleClick);
            #endregion
        }

        void Main_FormClosing(object sender, FormClosingEventArgs e)
        {
            Environment.Exit(0);
        }

        void listView1_DoubleClick(object sender, EventArgs e)
        {
            Entry E = (Entry)listView1.FocusedItem.Tag;
            foreach (TreeNode n in treeView1.SelectedNode.Nodes)
            {
                if (n.Text == E.Name)
                {
                    treeView1.SelectedNode = treeView1.SelectedNode.Nodes[n.Index];
                    ReadFolderData();
                }
            }
        }

        #region Treeview Editing
        /// <summary>
        /// Renames the folder on the drive - Checks if the node is a root/partition
        /// </summary>
        void treeView1_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            if (!Beta)
            {
                if (treeView1.SelectedNode != treeView1.Nodes[0])
                {
                    bool canEdit = true;
                    for (int i = 0; i < treeView1.Nodes[0].Nodes.Count - 1; i++)
                    {
                        if (treeView1.SelectedNode == treeView1.Nodes[0].Nodes[i])
                        {
                            canEdit = false;
                            e.CancelEdit = true;
                            return;
                        }
                    }
                    if (canEdit)
                    {
                        Folder f = (Folder)treeView1.SelectedNode.Tag;
                        Write w = new Write(xDrive);
                        bool result = w.Rename(f, e.Node.Text);
                        if (result)
                        {
                            treeView1.SelectedNode.Tag = f;
                        }
                        else
                        {
                            e.CancelEdit = true;
                        }
                    }
                }
                else
                {
                    e.CancelEdit = true;
                }
            }
            else
            {
                MessageBox.Show(" Function currently unavailable");
                e.CancelEdit = true;
            }
        }
        #endregion

        #region Column Sorting for Listview
        /// <summary>
        /// Column Sorter
        /// </summary>
        void listView1_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            //Check if the last column clicked was this column
            if (clickedColumn == e.Column)
            {
                //Check the current sorting
                if (listView1.Sorting == SortOrder.Ascending)
                {
                    listView1.Sorting = SortOrder.Descending;
                }
                else
                {
                    listView1.Sorting = SortOrder.Ascending;
                }
            }
            else
            {
                clickedColumn = e.Column;
                listView1.Sorting = SortOrder.Ascending;
            }
        }
        #endregion

        internal FATXDrive xDrive
        {
            get { return drive; }
            set { drive = value; }
        }

        #region Show Forms
        /// <summary>
        /// Show about form
        /// </summary>
        private void ShowAbout_menuItem7_Click(object sender, EventArgs e)
        {
            About abt = new About();
            abt.ShowDialog();
        }

        private void ShowDeviceSelector_menuItem2_Click(object sender, EventArgs e)
        {
            DeviceSelector ds = new DeviceSelector(this);
            if (ds.ShowDialog() == DialogResult.OK)
            {
                Loaded = true;
                xDrive.ReadData();
                ReadDrive();
            }
        }
        #endregion

        /// <summary>
        /// Clears and resets the form
        /// </summary>
        private void Clear()
        {
            foreach (MenuItem mi in listview_ContextMenu.MenuItems)
            {
                mi.Enabled = false;
            }
            foreach (MenuItem mi in treeview_ContextMenu.MenuItems)
            {
                mi.Enabled = false;
            }
            foreach (MenuItem mi in menuItem9.MenuItems)
            {
                mi.Enabled = false;
            }
            listView1.Items.Clear();
            treeView1.Nodes[0].Nodes.Clear();
            treeView1.Nodes[0].Text = "Drive";
            Loaded = false;
            if (xDrive != null)
            {
                xDrive.Close();
                xDrive = null;
            }
            pathBar.Text = "";
        }

        public void ReadDrive()
        {
            foreach (MenuItem mi in listview_ContextMenu.MenuItems)
            {
                if (mi.Text != "New Folder" && mi.Text != "Delete" && mi.Text != "Rename" && mi.Text != "Inject New File" && Beta)
                {
                    mi.Enabled = true;
                }
            }
            foreach (MenuItem mi in treeview_ContextMenu.MenuItems)
            {
                if (mi.Text != "New Folder" && mi.Text != "Delete" && mi.Text != "Rename" && mi.Text != "Inject New File" && Beta)
                {
                    mi.Enabled = true;
                }
            }
            foreach (MenuItem mi in menuItem9.MenuItems)
            {
                mi.Enabled = true;
            }

            if (xDrive.DriveType != Info.DriveType.HDD)
            {
                menuItem10.Enabled = false;
                menuItem11.Enabled = false;
            }
            AddPartitionsToTree();
        }

        /// <summary>
        /// Adds Base Partitions to treeview
        /// </summary>
        private void AddPartitionsToTree()
        {
            //Create our folders (that will act as partitions) for each partition.
            Folder[] Partitions = new FATX.Partitions().Get(xDrive);
            //Set their EData
            for (int i = 0; i < Partitions.Length; i++)
            {
                Partitions[i].Name = Partitions[i].PartInfo.Name;
                Partitions[i].EData.StartingCluster = 1;
            }

            //Clear the subnodes
            treeView1.Nodes[0].Nodes.Clear();
            treeView1.Nodes[0].Text = xDrive.DriveType.ToString() + xDrive.DriveSizeConverted;
            
            //Create our nodes
            List<TreeNode> TN = new List<TreeNode>();
            foreach (Folder f in Partitions)
            {
                TreeNode tn = new TreeNode(f.Name);
                tn.Tag = f;
                TN.Add(tn);
            }
            TreeNode[] nodeArray = TN.ToArray();
            treeView1.Nodes[0].Nodes.AddRange(nodeArray);
            treeView1.Nodes[0].Expand();
        }

        private void menuItem4_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// </summary>
        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            ReadFolderData();
        }

        private void ReadSubFolders(ref TreeNode node)
        {
            foreach (Folder f in ((Folder)node.Tag).SubFolders(Deletionmode))
            {
                TreeNode subNode = new TreeNode(f.Name);
                subNode.Tag = f;
                node.Nodes.Add(subNode);
            }
        }

        private void ReadFolderData()
        {
            //try
            //{
                if (Loaded && treeView1.SelectedNode != treeView1.Nodes[0])
                {
                    //If they just loaded this node
                    if (lastClickedNode == treeView1.SelectedNode)
                    {
                        return;
                    }
                    //Set the last clicked nocde
                    lastClickedNode = treeView1.SelectedNode;
                    //Clear our listview
                    listView1.Items.Clear();
                    //Create a list of our nodes to add
                    List<TreeNode> treeList = new List<TreeNode>();
                    //Create our itemlist as well
                    List<ListViewItem> itemList = new List<ListViewItem>();
                    //Create our reader
                    Folder fffffff = (Folder)lastClickedNode.Tag;
                    FATX.Entries read = new FATX_Browser.FATX.Entries(fffffff);
                    //Loop for each folder
                    Folder[] folders = ((Folder)lastClickedNode.Tag).SubFolders(Deletionmode);
                    foreach (FATX.Folder f in folders)
                    {
                        bool exists = false;
                        foreach (TreeNode TN in treeView1.SelectedNode.Nodes)
                        {
                            //If the node already exists...
                            if (TN.Text == f.Name)
                            {
                                //The node exists
                                exists = true;
                            }
                        }
                        if (!exists)
                        {
                            //Create a new node with the folder name
                            TreeNode node = new TreeNode(f.Name, 0, 0);
                            //Create our treenode subitems (subfolders) and add them to
                            //the node

                            ///To make loading faster, let's not load the subfolders.
                            //foreach (Folder g in f.SubFolders())
                            //{
                            //    TreeNode subNode = new TreeNode(g.Name);
                            //    subNode.Tag = g;
                            //    //ReadSubFolders(ref subNode);
                            //    node.Nodes.Add(subNode);
                            //}

                            //Set the tag
                            node.Tag = f;
                            //Add to the list
                            treeList.Add(node);
                        }

                        //Create our listview item
                        ListViewItem item = new ListViewItem(f.Name, 0);
                        //Set our tag
                        item.Tag = f;
                        //Create our "file type" subitem
                        ListViewItem.ListViewSubItem type = new ListViewItem.ListViewSubItem(item, f.EntryType);
                        ListViewItem.ListViewSubItem date = new ListViewItem.ListViewSubItem(item, new Misc().GetDateTimeFromNum(f.EData.CreationDate, f.EData.CreationTime).ToString());
                        item.SubItems.Add(type);
                        item.SubItems.Add("");
                        item.SubItems.Add(date);
                        //Add our item to the list
                        itemList.Add(item);
                    }
                    //For each file
                    File[] files = ((Folder)lastClickedNode.Tag).Files(Deletionmode);
                    foreach (File f in files)
                    {

                        //Create a new listview item with the file name
                        ListViewItem item = new ListViewItem(f.Name, 1);
                        //Set the tag
                        item.Tag = f;
                        //Create our "file type" subitem
                        ListViewItem.ListViewSubItem type = new ListViewItem.ListViewSubItem(item, f.EntryType);
                        //Create our "size" subitem
                        ListViewItem.ListViewSubItem size = new ListViewItem.ListViewSubItem(item, misc.ByteConversion(f.Size));
                        ListViewItem.ListViewSubItem date = new ListViewItem.ListViewSubItem(item, new Misc().GetDateTimeFromNum(f.EData.CreationDate, f.EData.CreationTime).ToString());
                        item.SubItems.Add(type);
                        item.SubItems.Add(size);
                        item.SubItems.Add(date);
                        //Add to the list
                        itemList.Add(item);
                    }
                    //Add our list of folders to the treeview node
                    treeView1.SelectedNode.Nodes.AddRange(treeList.ToArray());
                    //Add our list of files and folders to the listview
                    listView1.Items.AddRange(itemList.ToArray());
                    //Extend the node
                    treeView1.SelectedNode.Expand();
                    string[] ye = treeView1.SelectedNode.FullPath.Split('\\');
                    string pathString = "";
                    for (int i = 1; i < ye.Length; i++)
                    {
                        if (i == ye.Length - 1)
                        {
                            pathString += ye[i];
                        }
                        else
                        {
                            pathString += ye[i] + "\\";
                        }
                    }
                    pathBar.Text = pathString;
                }
            //}
            //catch { MessageBox.Show("Error reading from drive"); Clear(); }
        }

        private void RemoveDeleted()
        {
            foreach (ListViewItem i in listView1.Items)
            {
                Entry e = (Entry)i.Tag;
                if (e.IsDeleted)
                {
                    listView1.Items.Remove(i);
                }
            }

            RemoveFromTree(treeView1.Nodes[0]);
        }

        private void RemoveFromTree(TreeNode Node)
        {
            foreach (TreeNode n in Node.Nodes)
            {
                try
                {
                    Entry e = (Entry)n.Tag;
                    if (e.IsDeleted)
                    {
                        treeView1.Nodes.Remove(n);
                    }
                }
                catch { continue; }

                RemoveFromTree(n);
            }
        }

        private bool GoToPath(string path, string priorPath)
        {
            bool foundFolder = false;
            //Split our path in to an array (folder names)
            string[] folders = path.Split('\\');
            //Set our selected item to the partition node
            for (int i = 0; i < treeView1.Nodes[0].Nodes.Count; i++)
            {
                if (treeView1.Nodes[0].Nodes[i].Text.ToLower() == folders[0].ToLower())
                {
                    treeView1.SelectedNode = treeView1.Nodes[0].Nodes[i];
                    foundFolder = true;
                }
            }
            //If we didn't find the folder...
            if (foundFolder == false)
            {
                if (path == priorPath)
                {
                    return false;
                }
                GoToPath(priorPath, priorPath);
                //Return false
                return false;
            }
            foundFolder = true;
            //Foreach folder in our array
            foreach (string s in folders)
            {
                for (int i = 0; i < treeView1.SelectedNode.Nodes.Count; i++)
                {
                    if (treeView1.SelectedNode.Nodes[i].Text.ToLower() == s.ToLower())
                    {
                        treeView1.SelectedNode = treeView1.SelectedNode.Nodes[i];
                        foundFolder = true;
                    }
                }

                if (foundFolder == false)
                {
                    if (path == priorPath)
                    {
                        return false;
                    }
                    GoToPath(priorPath, priorPath);
                    return false;
                }

                foundFolder = false;
            }
            return true;
        }

        /// <summary>
        /// Extracts an object from the listview
        /// </summary>
        private void ExtractListview_menuItem14_Click(object sender, EventArgs e)
        {
            if (!((Entry)listView1.FocusedItem.Tag).IsFolder)
            {
                SaveFileDialog ofd = new SaveFileDialog();
                ofd.FileName = listView1.FocusedItem.Text;
                if (ofd.ShowDialog() == DialogResult.OK)
                {
                    Extraction ex = new Extraction((File)listView1.FocusedItem.Tag, ofd.FileName, Deletionmode);
                    ex.ShowDialog();
                }
            }
            else
            {
                FolderBrowserDialog fbd = new FolderBrowserDialog();
                if (fbd.ShowDialog() == DialogResult.OK)
                {
                    Extraction Ex = new Extraction((Folder)listView1.FocusedItem.Tag, fbd.SelectedPath, Deletionmode);
                    Ex.ShowDialog();
                }
            }
        }

        private void ExtractForm(object sender, EventArgs e)
        {
            TreeNode n = treeView1.SelectedNode;
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            if (fbd.ShowDialog() == DialogResult.OK)
            {
                Extraction Ex = new Extraction((Folder)n.Tag, fbd.SelectedPath, Deletionmode);
                Ex.ShowDialog();
            }
        }

        /// <summary>
        /// Opens dialog to allow user to save backup of drive
        /// </summary>
        private void ShowBackup_menuItem10_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Filter = "Backup File(*.bin)|*.bin";
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                Backup b = new Backup(sfd.FileName, xDrive);
                b.ShowDialog();
            }
        }

        private void menuItem22_Click(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode != treeView1.Nodes[0])
            {
                bool canEdit = true;
                for (int i = 0; i < treeView1.Nodes[0].Nodes.Count; i++)
                {
                    if (treeView1.SelectedNode == treeView1.Nodes[0].Nodes[i])
                    {
                        canEdit = false;
                        return;
                    }
                }
                if (canEdit)
                {
                    treeView1.SelectedNode.BeginEdit();
                }
            }
        }

        private void menuItem21_Click(object sender, EventArgs e)
        {

        }

        /// <summary>
        /// Opens drive backup
        /// </summary>
        private void OpenBackup_menuItem16_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                Clear();
                xDrive = new FATXDrive(ofd.FileName, Info.DriveType.Backup);
                Loaded = true;
                xDrive.ReadData();
                ReadDrive();
            }
        }

        private void menuItem3_Click(object sender, EventArgs e)
        {
            Clear();
        }

        private void NodeFromPath()
        {

        }

        private void menuItem18_Click(object sender, EventArgs e)
        {
            if (!Beta)
            {
                Write w = new Write(xDrive);
                if (listView1.FocusedItem != null && ((Entry)listView1.FocusedItem.Tag).IsFolder)
                {
                    Folder f = (Folder)listView1.FocusedItem.Tag;
                    long throwaway = 0;
                    long bleeh = 0;
                    string currentE = "";
                    w.Delete(f, ref throwaway, ref bleeh, ref currentE);
                    foreach (TreeNode n in treeView1.SelectedNode.Nodes)
                    {
                        if (n.Text == f.Name)
                        {
                            treeView1.SelectedNode.Nodes.Remove(n);
                            break;
                        }
                    }
                }
            }
            else
            {
                MessageBox.Show(" Function currently unavailable");
            }
        }

        private void menuItem26_Click(object sender, EventArgs e)
        {
            if (!Beta)
            {
                Folder f = (Folder)treeView1.SelectedNode.Tag;
                f.NewFolder(GetNewFolderName(f));
                f.ReloadData(Deletionmode);
                treeView1.SelectedNode.Tag = f;
                ReadFolderData();
            }
            else
            {
                MessageBox.Show(" Function currently unavailable");
            }
        }

        private string GetNewFolderName(Folder f)
        {
            int FoldersFound = 0;
            string NewName = "New Folder";
            foreach (Folder g in f.SubFolders(false))
            {
                if (g.Name == NewName)
                {
                    FoldersFound++;
                    NewName = "New Folder (" + FoldersFound.ToString() + ")";
                }
            }
            return NewName;
        }

        private void menuItem23_Click(object sender, EventArgs e)
        {

        }

        private void button1_Click(object sender, EventArgs e)
        {
            string newText = pathBar.Text;
            pathBar.Undo();
            string oldText = pathBar.Text;
            pathBar.Text = newText;
            if (!GoToPath(newText, oldText))
            {
                pathBar.Text = oldText;
                MessageBox.Show("Path not found");
            }
        }

        private void menuItem23_Click_1(object sender, EventArgs e)
        {
            FATX_Browser.Update u = new FATX_Browser.Update();
            FATX_Browser.Update.UpdateInfo ui;
            ui = u.CheckForUpdates(new Uri("http://clkxu5.com/drivexplore/coolapplicationstuff.xml"));
            if (ui.Update)
            {
                UpdateForm uf = new UpdateForm(ui);
                if (uf.ShowDialog() == System.Windows.Forms.DialogResult.Yes)
                {
                    UpdateDownloadForm ud = new UpdateDownloadForm(ui);
                    ud.ShowDialog();
                }
            }
            else if (ui.Update == false && ui.UpdateVersion == Application.ProductVersion)
            {
                MessageBox.Show("Party Buffalo Drive Explorer is up to date!  Thanks for flying Buffalo Airlines.");
            }
            else
            {
                MessageBox.Show("Looks like you might not be connected to the internet, or your firewall is blocking this app from reaching the server.  Oh well, updates -- who needs 'em?");
            }
        }

        private void DisableWritingFunctions()
        {
            foreach (MenuItem mi in listview_ContextMenu.MenuItems)
            {
                if (mi.Text != "New Folder" && mi.Text != "Delete" && mi.Text != "Rename" && mi.Text != "Inject New File")
                {
                    mi.Enabled = true;
                }
                else
                {
                    mi.Enabled = false;
                }
            }
            foreach (MenuItem mi in treeview_ContextMenu.MenuItems)
            {
                if (mi.Text != "New Folder" && mi.Text != "Delete" && mi.Text != "Rename" && mi.Text != "Inject New File")
                {
                    mi.Enabled = true;
                }
                else
                {
                    mi.Enabled = false;
                }
            }
        }

        private void EnableWritingFunctions()
        {
            foreach (MenuItem mi in listview_ContextMenu.MenuItems)
            {
                if (mi.Text != "New Folder" && mi.Text != "Delete" && mi.Text != "Rename" && mi.Text != "Inject New File")
                {
                    mi.Enabled = true;
                }
                else
                {
                    mi.Enabled = true;
                }
            }
            foreach (MenuItem mi in treeview_ContextMenu.MenuItems)
            {
                if (mi.Text != "New Folder" && mi.Text != "Delete" && mi.Text != "Rename" && mi.Text != "Inject New File")
                {
                    mi.Enabled = true;
                }
                else
                {
                    mi.Enabled = true;
                }
            }
        }

        private void menuItem29_Click(object sender, EventArgs e)
        {
            if (DeletionMode.Checked == true)
            {
                DeletionMode.Checked = false;
                Deletionmode = false;
                RemoveDeleted();
                return;
            }
            Deletionmode = true;
            DeletionMode.Checked = true;
        }
    }
}
