﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Net;
using System.IO;

namespace FATX_Browser
{
    public class Update
    {
        [DllImport("wininet.dll")]
        private extern static bool InternetGetConnectedState( out int Description, int ReservedValue ) ;

        public struct UpdateInfo
        {
            public bool Update;
            public bool DLLNeeded;
            public string UpdateVersion;
            public string UpdateText;
            public string UpdatePath;
            public string UpdateDLLPath;
        }

        public bool CheckInternetConnection()
        {
            int Desc;
            return InternetGetConnectedState(out Desc, 0);
        }


        public UpdateInfo CheckForUpdates(Uri relativeURL)
        {
            UpdateInfo info = new UpdateInfo();
            info.Update = false;
            string[] CurrentVersion = new string[4];
            //To see if the user is connected to the internet...
            if (!CheckInternetConnection())
            {
                //return they have no internet connection
                return info;
            }
            try
            {
                System.Xml.XmlTextReader r = new System.Xml.XmlTextReader(relativeURL.ToString());
                while (r.Read())
                {
                    if (r.NodeType == System.Xml.XmlNodeType.Element)
                    {
                        if (r.Name == "CurrentVersion")
                        {
                            info.UpdateVersion = r.ReadInnerXml();
                            CurrentVersion = info.UpdateVersion.Split('.');
                            string[] AppVersion = System.Windows.Forms.Application.ProductVersion.Split('.');

                            int[] Current = new int[4];
                            int[] App = new int[4];
                            for (int i = 0; i < Current.Length; i++)
                            {
                                Current[i] = Convert.ToInt32(CurrentVersion[i]);
                                App[i] = Convert.ToInt32(AppVersion[i]);
                            }

                            if (Current[0] > App[0])
                            {
                                info.Update = true;
                                return info;
                            }
                            if (Current[1] > App[1])
                            {
                                info.Update = true;
                                return info;
                            }
                            if (Current[2] > App[2])
                            {
                                info.Update = true;
                                return info;
                            }
                            if (Current[3] > App[3])
                            {
                                info.Update = true;
                                return info;
                            }
                            return info;
                        }

                        if (r.Name == "UpdateFixes")
                        {
                            info.UpdateText = r.ReadInnerXml();
                        }

                        if (r.Name == "UpdatePath")
                        {
                            info.UpdatePath = r.ReadInnerXml();
                        }

                        if (r.Name == "UpdateDLLPath")
                        {
                            info.UpdateDLLPath = r.ReadInnerXml();
                        }

                        if (r.Name == "DLLNeeded")
                        {
                            if (r.ReadInnerXml() == "true")
                            {
                                info.DLLNeeded = true;
                            }
                            else
                            {
                                info.DLLNeeded = false;
                            }
                        }
                    }
                }
            }
            catch { return info; }
            return info;
        }

        public bool DownloadUpdate(UpdateInfo ui, System.Windows.Forms.Form sender)
        {
            try
            {
                //Get our array of each part of the full path so we can get the file name
                string[] updateSplit = ui.UpdatePath.Split('/');
                //This will get the new exe's name
                string updateName = updateSplit[updateSplit.Length - 1].Remove(updateSplit[updateSplit.Length - 1].Length - 4) + "`.exe";
                //Do the same thing for the dll
                string[] dllSplit = ui.UpdateDLLPath.Split('/');
                string dllName = dllSplit[dllSplit.Length - 1].Remove(dllSplit[dllSplit.Length - 1].Length - 4) + ".dll";
                //Create our new web client so we can download the file
                System.Net.WebClient wc = new System.Net.WebClient();
                //Download the exe
                if (System.IO.File.Exists(updateName))
                {
                    System.IO.File.Delete(updateName);
                }
                wc.DownloadFile(ui.UpdatePath, updateName);
                //If the dll does not exist
                if (!System.IO.File.Exists(System.Windows.Forms.Application.StartupPath + "\\" + dllName) && ui.DLLNeeded)
                {
                    //Download that as well
                    wc.DownloadFile(ui.UpdateDLLPath, dllName);
                }
                //Start the new update, passing our arguments to the old executable's path
                //so it can be deleted
                System.Diagnostics.Process.Start(System.Windows.Forms.Application.StartupPath + "\\" + updateName, System.Windows.Forms.Application.ExecutablePath);
                //Close the app
                Environment.Exit(0);
            }
            catch (Exception e) { System.Windows.Forms.MessageBox.Show("Whoa!  We've encountered a problem.  Please report this error!\r\n" + e.Message); sender.Invoke(new CloseForm(sender.Close)); return false; }
            return true;
        }

        private delegate void CloseForm();
    }
}
